# -*- coding: utf-8 -*-
"""
CREATED
    @author: Morten Lybech Thøgersen, 2023-05-22 (mlt@emd.dk)

PURPOSE:
    An example on howto get status from the EMD-API: EMD-WRF On-Demand -
    as a minimal example using the python requests package.

    This file is a part of set of 3 example files each with a specific focus:
       1. Status and overview of user-request and past-calculations (emdwrfod_example_status.py)
       2. Submit a calculation (emdwrfod_example_order_submit.py)
       3. Check status and retrieve data from one or more calculations (emdwrfod_example_order_check.py)

USAGE:
    1. Obtain a API access token from EMD (contact sales@emd.dk)
    2. Purchase some calculation-credits (1 credit is worth 1 month of mesoscale-timeseries for the basic configuration)
    3. Run this script and inspect the results

REFERENCES:
    .mesores documentation with samples:
       https://help.emd.dk/knowledgebase/
       https://www.emd.dk/files/flow/EMD_technote_MESORES_20230426.pdf
    UI: https://api.emd.dk/emd-wrf-od/ui

"""

import pandas as pd
import requests
import json
import tabulate
import sys


if __name__ == '__main__':

    # Input data: API-URL and User-Token
    api_url = "https://api.emd.dk/emd-wrf-od"
    user_token = '16d5e6de-74da-4d03-a06c-ee1942417959' # EMD-API OD TEST USER

    # Get list of all available datasets for this user account
    response_datasets = requests.get(api_url+"/models", headers={'X-Auth': user_token}, verify=True)
    if response_datasets.status_code == 401:
        print("Please obtain a valid user token by contacting EMD: ", response_datasets.status_code)
        sys.exit()
    df_datasets = pd.read_json(response_datasets.content)
    df_datasets = df_datasets[['description', 'model_id', 'start_date', 'end_date', 'supported_areas']]
    print("\nEMD-API AVAILABLE MODELS")
    print("------------------------")
    print(tabulate.tabulate(df_datasets, headers=df_datasets.columns, showindex=False))

    # Get information about used and purchased credits for this user-account
    response_info = requests.get(api_url + "/info", headers={'X-Auth': user_token}, verify=True)
    info_status = json.loads(response_info.content)
    print("\nACCOUNT INFORMATION - CREDITS OVERVIEW")
    print("--------------------------------------")
    print("   Purchased: ", info_status['credits_purchased'])
    print("   Remaining: ", info_status['credits_remaining'])
    print("   Used: ", info_status['credits_used'])

    print("\nACCOUNT INFORMATION - PAYMENTS")
    print("------------------------------")
    df_payments = pd.DataFrame(data=info_status['payments'])
    print(tabulate.tabulate(df_payments, headers=df_payments.columns, showindex=False))

    print("\nACCOUNT INFORMATION - CALCULATIONS")
    print("----------------------------------")
    df_calculations = pd.DataFrame(data=info_status['calculations'])
    if not(df_calculations.empty):
        df_calculations = df_calculations[['datetime', 'status', 'user', 'credits', 'order_id', 'configuration']]
        df_calculations = df_calculations.sort_values('datetime', ascending=False)
    print(tabulate.tabulate(df_calculations, headers=df_calculations.columns, showindex=False))

    # List all previous orders and obtain user-id
    print("\nACCOUNT INFORMATION - STATUS OF ALL PREVIOUS ORDERS")
    print("---------------------------------------------------")
    status_list = []
    for idx, row in df_calculations.iterrows():
        orderid = row["order_id"]
        try:
            response_status = requests.get(api_url + "/status/" + row["order_id"], headers={'X-Auth': user_token}, verify=True)
            order_status = json.loads(response_status.content)
            status_list.append([row["order_id"], order_status["status"], order_status["download_url"], order_status["estimated_completion_time"], order_status["message"]])
        except KeyError as e:
            if response_status:
                print(response_status.content)
    df_status = pd.DataFrame(data=status_list, columns=["order_id", "status", "download_url", "estimated_completion_time", "message"])
    print(tabulate.tabulate(df_status, headers=df_status.columns, showindex=False))
